use anyhow::{Context, Result};
use std::{fs::File, io::Write, path::Path};

pub struct FetchResult {
    pub cached: bool,
    pub html: String,
}

pub fn fetch_cache(
    work_dir: &str,
    client: &reqwest::blocking::Client,
    url: &str,
) -> Result<FetchResult> {
    let (_, filename) = url.rsplit_once('/').context("Failed to split URL.")?;
    let path = Path::new(work_dir).join(filename);

    if path.exists() {
        // File already exists -- use cached version.
        let buffer = std::fs::read_to_string(&path)
        .with_context(|| {
            format!("Failed to read file [{:?}].", &path)
            // path.to_string_lossy().to_string());
        })?;

        return Ok(FetchResult {
            cached: true,
            html: buffer,
        });
    }

    println!("fetch_cache {} => {}", url, filename);
    let res = client.get(url).send().unwrap();
    let buffer = res.text().unwrap();

    let mut file =
        File::create(&path)
        .with_context(|| { 
            format!("Failed to write file [{:?}].", &path) 
        })?;

    file.write_all(buffer.as_bytes())
        .with_context(|| {
            format!("Failed to write all to [{:?}].", &path)
        })?;

    Ok(FetchResult {
        cached: false,
        html: buffer,
    })
}
